# NPM Webpack

### NOTE TO SELF, MAKE SURE TO HAVE THE IMAGES FOLDER READY FOR THE STUDENTS TO ADD

We'll use a tool called webpack to explore different ways to make and use modules in JavaScript

Create a folder for this project named **npm-webpack**
Add a **public** folder to the npm-webpack folder

In the public folder, add a file named **index.html** and put this code in it:

```html
<!DOCTYPE html>
<html>
<head>
	<meta charset="utf-8">
	<title>Web Pack</title>
</head>
<body>
	<script src="bundle.js"></script>
</body>
</html>
```
Note that it's linking to a file named bundle.js, which we did not create.
Webpack will create this file for us.

Add a **src** folder to the npm-webpack folder, this is where we'll put all of our source code

In the src folder add a file named **index.js**, leave it empty for the moment.

### Create Class Module
In the src folder add a file named **class-module.js** and put this code in it:
```js
class SomeClass{

	someMethod(){
		alert("Some method is running")
	}

	someOtherMethod(){
		alert("Some other method is running")
	}

}

export default SomeClass

```
Note that this file defines a JavaScript class. 
Also note that the class is being exported (at the bottom).
This will allow us to import it into other files.

Go back to index.js and add this code:
```js
import SomeClass from './class-module'

const obj = new SomeClass();
obj.someMethod();
```
We import SomeClass, then instantiate it and invoke a method.



# START HERE
Clone the repository and then open the folder in VSCode.

Take a look a the files and folders in the project.
Notice that the index.html file is in the 'public' folder, and that it links to a file javascript file named 'bundle.js' that does not exist. All the other files for the project are in the src folder.

### Setup Webpack
At this point, we can't run this code, so we'll use Webpack to compile it into the **public** folder.
Open a terminal in the npm-webpack folder and initialize an npm project by running this command:
```md
npm init -y
```
Note that the **-y** will create the package.json file with default settings so that we don't have
to anser a bunch of questions to set everything up.

Now install two webpack packages by running this command:
```md
npm install webpack webpack-cli --save-dev
``` 

In the npm-webpack folder, create a file named **webpack.config.js** and add this to it:
```js
const path = require('path')

module.exports = {
	mode: 'development',
	entry: './src/js/index.js',
	output: {
		path: path.resolve(__dirname, 'public'),
		filename: 'bundle.js'
	},
	devtool:false,
}

```
This file specifies index.js to be the entry point, meaning that it's the file that will 
import all of our javascript modules.
The output object tells webpack to 'bundle' all of our javascript code into a file named bundle.js
and to put it in the public folder.

The 'mode' is set to 'development', we could change this to 'production' to compress our code,
but we'll leave it as is for now.

By setting the devtool property to false, we are disabling source maps, which can make your compiled code very difficul to read!
It you want to experiment with this, set devtool to 'source-map' 

Open a terminal in the project folder (npm-webpack) and run this command:
```md
npx webpack
```
You should see a bundle.js file in the public folder.

At this point you should be able to open the index.html page in the browser and see your JavaScript code run.

If you change the mode in webpack.config.js to 'production' and run webpack again, you'll see that bundle.js will be much smaller.

### Create a Functions Module
Lately there's been a trend toward creating modules that consist of functions rather than
a class. This allows you to import only the functions you need to use (when
you import a class it will come with all of it's methods even if you don't need them all)

In the src folder create a file named **functions-module.js** and put this code in it:
```js
export function someFunction(){
	alert("someFunction() is running..");
}

export function someOtherFunction(){
	alert("someOtherFunction() is running..");
}
```
Note that it exports two functions.

Here is a slightly different way to write the module so that you declare what it exports at the bottom
(this version will work exactly the same as the one above, but I'm showing it to you in case you see it out in other people's code):
```js
function someFunction(){
	alert("someFunction() is running..");
}

function someOtherFunction(){
	alert("someOtherFunction() is running..");
}

export {someFunction, someOtherFunction}
```

Now update index.js to look like this:
```js
import SomeClass from './class-module'
import {someFunction} from './functions-module'

const obj = new SomeClass();
obj.someMethod();

someFunction();
```
Note that we only imported one of the functions from functions-module.js

Go ahead and run webpack to compile the JavaScript code into bundle.js.





## Set Up Webpack to Watch for Changes
It's gets old to have to run webpack when you make a change.
We can automate it by adding to the scripts object in the package.json file.
Add a 'build' property to the scripts object and set it's value like so:
```md
"build": "webpack --watch"
```
Now, in the terminal you can launch webpack and have it re-run whenever you save a file by running this command:
```md
npm run build
```


## Install a Dev Server
Install the webpack dev server package by running this command:
```md
npm install --save-dev webpack-dev-server
```
By default, webpack will look in the public folder for an index.html file to launch in the server.

Now add a 'serve' script to the package.json file (just under the 'build' script we added earlier)
```md
"serve": "webpack serve --port 8080 && webpack --watch"
```
Run this command in the terminal (from the project folder):
```md
npm run serve
```
Now you can work on your project and webpack will automatically watch for changes and reload the server!




///LEFT OFF HERE
## Adding CSS
In the src folder, create a file named **styles.css** and put this code into it:
```css
body{
	background-color: gray;
}
```

Install this package:
```md
npm install --save-dev style-loader css-loader
```
Webpack has many 'loader' packages that help you accomplish tasks.
After you install a loader, you usually need to configure it in the webpack.config.js file
by adding a 'module' object and a 'rules' array within it.

Add this code to the webpack.config.js file (put it inside the module.exports object):
```md
module: {
	rules: [
  		{
    		test: /\.css$/i,
    		use: ['style-loader', 'css-loader'],
  		},
	],
}
```
The value of the 'test' property is a regular expression that tells webpack to apply the two loaders to .css files.

The two loaders that we just installed will allow us to incorporate our CSS code via JavaScript,
Seems strange, but to do this import the stylesheet into index.js like so:
```js 
import './styles.css'
````

## Adding Images
When we first installed webpack, it included a built-in 'asset' loader that we
can use to incorporate images into our web page, we just need to configure it by adding 
another object to the 'rules' array in webpack.config.js:
```js
{
	test: /\.(png|svg|jpg|jpeg|gif)$/i,
	type: 'asset/resource',
}
```
Now import an image into index.js like so:
```js
import Koala from './images/Koala.jpg'
```
And add this code to the bottom of index.js:
```js
const koalaImg = new Image();
koalaImg.src = Koala;
document.body.append(koalaImg);
```

### Loading Other Assets
For more info on loading various assets, [check out this link](https://webpack.js.org/guides/asset-management/)

This might be worth a look:
https://dev.to/antonmelnyk/how-to-configure-webpack-from-scratch-for-a-basic-website-46a5

# Here is where we can end things - the idea was to talk about JS modules!!!!
# HERE IS WHERE THINGS STARTED TO FALL APART
I couldn't figure out how to copy index.html from the public folder to the dist folder
using the file-loader!!!!!!!!!

## Creating a 'dist' folder
Let's create a separate webpack config file that will create a dist folder that can be 
deployed to a live server.

We'll need to install a loader that will allow us to copy files into the dist folder:
```md
npm install file-loader --save-dev
```

Create a file in the project folder named **webpack.prod.js** and put this code in it:
```js
const path = require('path')

module.exports = {
	mode: 'production',
	entry: './src/index.js',
	output: {
		path: path.resolve(__dirname, 'dist'),
		filename: 'bundle.js'
	},
	module: {
	    rules: [
	      {
	        test: /\.(png|jpe?g|gif|svg|html|css)$/i,
	        use: [
	          {
	            loader: 'file-loader',
	          },
	        ],
	      },
	    ],
	}
}
```
Note that we're specifying the 'dist' folder the output.


Now update the 'build' script in package.json to look like this:
```js
"build": "webpack --config webpack.prod.js",
```
Now run this command to start the build script:
```md
npm run build
```


# Maybe we should just quit here, don't really need to go any further at this point





### Install Babel
We can transpile our modern JavaScript code into 'classic' code by using a few Bable packages.
Install them by running this command:
```md
npm install --save-dev @babel/core @babel/cli @babel/preset-env @babel/plugin-syntax-class-properties
```
Now add a file to the project folder named **babel.config.js** and put this in it:
```js
module.exports = {
    "presets": [
        [
            "@babel/preset-env",
            {
                "targets": {
                    "browsers": "ie > 4" // Just added this to test if it would actually change anything
                }
            }
        ]
    ],
    "plugins":  ["@babel/plugin-syntax-class-properties"]
}
```
This configures Babel to transpile or modern JS code to classic JS that will run on very old browsers (all the way back to IE 4!)
It also tells Babel to make use of the 'class-syntax-properties' plugin. Since we
are using classes (which are relatively new in JS) we need to transpile them into old school JS as well.

We'll also need to install one more package so that we can integrate Babel into our Webpack build routine.
```md
npm install --save-dev babel-loader
```
Update webpack.config.js by adding the following object to the 'rules' array:
```md
{
	test: /\.js$/,
	exclude: /node_modules/,
	use:{
		loader: babel-loader
	}
}

```
The object in the rules array indicates that we want to use the loader on .js files (that's a regular express),
but to exclude any files in the node_modules folder.

Now add some modern JavaScript to index.js. Add this code to the end of the file:
```js
// array destructuring (es6)
const [firstName, lastName] = ["Bob", "Smith"];

// object destructuring (es6)
const dog = {name:"Brutus", age:7}
const {name, age} = dog;

// The includes() method is a newer one
const letters = ["a","b","c"];
if(letters.includes("b")){
	console.log("yup!")
}
```
Now go ahead and run webpack.

### Problem
If you take a good look at the compiled code, you'll see that the includes() method did not get replaced!
There is some confusion about this out on the web!


##Setting Up a Multipage Site
```md
npm install --save-dev html-webpack-plugin
```






